""" SkaceKamen (c) 2015-2023 """
# pyinstaller -F main.py
#####################################################################
# imports
import os
import sys
import wot
#####################################################################
# Add libs to path
wd = os.path.dirname(os.path.realpath(__file__))
sys.path.insert(0, os.path.join(wd, 'lib'))
#####################################################################
# Supported formats and their classes
supported_formats = {
	'obj': wot.OBJModelWriter,
	'collada': wot.ColladaModelWriter
}
#####################################################################
# Load default options
scale = [1, 1, 1]
transform = [1, 1, 1]
compress = False
output_material = True
output_vt = True
output_vn = True
textures_path = ''
writer_class = wot.OBJModelWriter
#####################################################################
# Load each input file
for filename_primitive in sys.argv[1:]:
	# Filter name
	filename = os.path.splitext(filename_primitive)
	if filename[1] not in ['.primitives', '.primitives_processed']:
		continue
	# Print progress if allowed
	print('\nProcessing %s' % filename[0])
	filename_visual = '%s.visual' % filename[0]
	# Guess correct names
	if filename_primitive.endswith('_processed'):
		filename_visual += '_processed'
	filename_obj = '%s%s' % (filename[0], writer_class.ext)
	filename_mtl = '%s.mtl' % filename[0]
	# Check for existence
	for fpath in (filename_primitive, filename_visual):
		if not os.path.exists(fpath):
			input('■ Failed to find %s' % fpath)
			continue
	# Intialize readers and writers
	model_reader = wot.ModelReader()
	model_writer = writer_class(
		compress=compress,
		normals=output_vn,
		uv=output_vt,
		material=output_material,
		scale=scale,
		textureBase=textures_path
	)
	# Read and write at same time
	with open(filename_primitive, 'rb') as primitives_f:
		with open(filename_visual, 'rb') as visual_f:
			model_writer.write(model_reader.read(primitives_f, visual_f), filename_obj, filename_mtl)
#####################################################################
input('\nDone!')